<?php
namespace Teknisa\CUS\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class Cus00200ProdutosComPrecoDesatualizado extends \Zeedhi\Framework\Controller\Report {

    protected $novoCodigo;
    protected $environment;
    protected $connection;
    protected $consplan;
    protected $servicoPorGrupo;
    protected $qr2Strategy;
    protected $nrorg;
    protected $cdfilial;
    protected $nrConsPlan;
    protected $reportService;
    protected $birtStrategy;

    public function __construct($reportService, $environment, $novoCodigo, $connection, $consplan, $qr2Strategy, $servicoPorGrupo, $birtStrategy) {

        $this->environment               = $environment;
        $this->novoCodigo                = $novoCodigo;
        $this->connection                = $connection;
        $this->consplan                  = $consplan;
        $this->qr2Strategy               = $qr2Strategy;
        $this->servicoPorGrupo           = $servicoPorGrupo;
        $this->reportService             = $reportService;
        $this->birtStrategy              = $birtStrategy;

        parent::__construct($this->reportService);
    }

    protected $reportMapping = array(
        'CUS00200' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'CDFILIAL'    => 'P_CDFILIAL',
                'NRCONSPLAN'  => 'P_NRCONSPLAN',
                'DATAINIC'    => 'P_DATAINIC',
                'DATAFINA'    => 'P_DATAFINA',
                'DTREFE'      => 'P_DTREFE',
                'PERIINIC'    => 'P_PERIINIC',
                'PERIFINA'    => 'P_PERIFINA',
                'DTPERIREFE'  => 'P_DTPERIREFE',
                'IDCUSTOZERO' => 'P_IDCUSTOZERO',
                'NRORG'       => 'P_NRORG'
            )
        ),
        'CUS00201' => array(
            'strategy' => 'QR2',
            'parameterMapping' => array(
                'CDFILIAL'    => 'P_CDFILIAL',
                'NRCONSPLAN'  => 'P_NRCONSPLAN',
                'DATAINIC'    => 'P_DATAINIC',
                'DATAFINA'    => 'P_DATAFINA',
                'DTREFE'      => 'P_DTREFE',
                'PERIINIC'    => 'P_PERIINIC',
                'PERIFINA'    => 'P_PERIFINA',
                'DTPERIREFE'  => 'P_DTPERIREFE',
                'IDCUSTOZERO' => 'P_IDCUSTOZERO'
            )
        ),
        'CUS00202' => array(
            'strategy' => 'QR2',
            'parameterMapping' => array(
                'CDFILIAL'    => 'P_CDFILIAL',
                'NRCONSPLAN'  => 'P_NRCONSPLAN',
                'DATAINIC'    => 'P_DATAINIC',
                'DATAFINA'    => 'P_DATAFINA',
                'DTREFE'      => 'P_DTREFE',
                'PERIINIC'    => 'P_PERIINIC',
                'PERIFINA'    => 'P_PERIFINA',
                'DTPERIREFE'  => 'P_DTPERIREFE',
                'IDCUSTOZERO' => 'P_IDCUSTOZERO'
            )
        ),
        'CUS00203' => array(
            'strategy' => 'QR2',
            'parameterMapping' => array(
                'CDFILIAL'    => 'P_CDFILIAL',
                'NRCONSPLAN'  => 'P_NRCONSPLAN',
                'DATAINIC'    => 'P_DATAINIC',
                'DATAFINA'    => 'P_DATAFINA',
                'DTREFE'      => 'P_DTREFE',
                'PERIINIC'    => 'P_PERIINIC',
                'PERIFINA'    => 'P_PERIFINA',
                'DTPERIREFE'  => 'P_DTPERIREFE',
                'IDCUSTOZERO' => 'P_IDCUSTOZERO'
            )
        ),
        'CUS00204' => array(
            'strategy' => 'QR2',
            'parameterMapping' => array(
                'CDFILIAL'    => 'P_CDFILIAL',
                'NRCONSPLAN'  => 'P_NRCONSPLAN',
                'DATAINIC'    => 'P_DATAINIC',
                'DATAFINA'    => 'P_DATAFINA',
                'DTREFE'      => 'P_DTREFE',
                'PERIINIC'    => 'P_PERIINIC',
                'PERIFINA'    => 'P_PERIFINA',
                'DTPERIREFE'  => 'P_DTPERIREFE',
                'IDCUSTOZERO' => 'P_IDCUSTOZERO'
            )
        )
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $row['NRORG'] = $this->environment->getNrOrg();
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));

        $cdgrupserv = $row['LISTAGRUPSERV'];
        $cdfilial   = $row['CDFILIAL'];
        $cdoperador = $this->environment->getCdOperador();
        $nrorg      = $this->environment->getNrOrg();

        $dateInicial  = $row['DTINICIAL'];
        $dateFinal    = $row['DTFINAL'];

        if ($row['LISTASERV'] == 'REQUEST') {
            $row['LISTASERV'] = $this->servicoPorGrupo->retornaServicosPorGrupo($nrorg, $cdfilial, $cdoperador, $cdgrupserv);
        }

        $nrConsPlan = $this->executaProcedure($cdfilial, $row['LISTASERV'], $cdoperador, $nrorg, $dateInicial, $dateFinal);
        $this->montaRow($row, $nrConsPlan, $dateInicial, $dateFinal);
    }

    public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report) {
        $parametros  = array(
            "NRORG" => $this->nrorg,
            "CDFILICPLA" => $this->cdfilial,
            "NRCONSPLAN" => $this->nrConsPlan,
            "__is_new" => false
        );
        $newRequest  = new DTO\Request\Row($parametros, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        $newResponse = new DTO\Response();
        $this->consplan->delete($newRequest, $newResponse);
    }

    public function montaRow($row, $nrConsPlan, $dateInicial, $dateFinal) {
        $row['NRCONSPLAN'] = $nrConsPlan;
        $row['DATAINIC']   = $dateInicial;
        $row['DATAFINA']   = $dateFinal;
        $row['PERIINIC']   = $dateInicial;
        $row['PERIFINA']   = $dateFinal;
        $row['DTPERIREFE'] = $row['DTREFE'];
    }

    public function executaProcedure($cdfilial, $listaserv, $cdoperador, $nrorg, $dateInicial, $dateFinal) {
        $idArquivo    = "T";
        $idProdutos   = "CFP";
        $idSepPrato   = "P";
        $peAcrescimo  = 0;
        $idarredond   = "N";
        $cdProdutoIni = " ";
        $cdProdutoFin = "zzzzzzzzzz";
        
        $listaserv = explode("_", $listaserv);
        $listaserv = array_chunk($listaserv, 165);

        $dateInicialFormatted = \DateTime::createFromFormat('d/m/Y', $dateInicial);
        $dateFinalFormatted   = \DateTime::createFromFormat('d/m/Y', $dateFinal);

        $nrConsPlan = $this->novoCodigo->getCodeByTableName('CONSPLAN' . $cdfilial, $nrorg);
        foreach($listaserv as $key => &$listaServ){
            $listaServico = implode("_", $listaServ);
            $procedure = new StoredProcedure($this->connection, 'CONSUMO_PLANEJADO_PLA');
            $procedure->addParam(new Param('P_FILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_SERVICOS', Param::PARAM_INPUT, $listaServico, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, $dateInicial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT, $dateFinal, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDARQUIVO', Param::PARAM_INPUT, $idArquivo, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDPRODUTOS', Param::PARAM_INPUT, $idProdutos, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDSEPPRATO', Param::PARAM_INPUT, $idSepPrato, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_PEACRESCIMO', Param::PARAM_INPUT, $peAcrescimo, Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_IDARREDOND', Param::PARAM_INPUT, $idarredond, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCONSPLAN', Param::PARAM_INPUT, $nrConsPlan, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, $cdProdutoIni, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, $cdProdutoFin, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_STDTINICIAL', Param::PARAM_INPUT, "_" . $dateInicialFormatted->format('Ymd') . "_", Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_STDTFINAL', Param::PARAM_INPUT, "_" . $dateFinalFormatted->format('Ymd') . "_", Param::PARAM_TYPE_STR));
    
            $resultado = $procedure->execute();
        }

        return $nrConsPlan;
    }
}